@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (0, 6, 0)
-CODENAME = "CHRISTMAAAAAS"
-BUILD_DT = (2020, 12, 1)
+VERSION = (0, 7, 0)
+CODENAME = "keeping track"
+BUILD_DT = (2021, 1, 10)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
